package genObserver;

import genObserver.IModel.Uhrzeit;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JTextField;


import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.EnumMap;
import java.util.Map;

/**
 * Uhr mit MVC Observer - View fr digitale Anzeige
 * Ist Observer, lauscht auf die Ereignisse vom Control
 * @author Prof. Solymosi, Beuth-Hochschule fr Technik Berlin
 * @version 16.4.2012
 */

@SuppressWarnings("serial")
public class DigitalView extends JFrame implements Observer<Ereignis> {
	final static String DIGITAL = "Digitaluhr: "; // Beschriftung		
	final static String TITEL = "Uhr";
	private static final int BREITE = 400, HHE = 300;	// Gre des Frames

	private IModel model;	
	private JTextField textField;	
	private Map<Ereignis, JButton> knpfe = new EnumMap<Ereignis, JButton>(Ereignis.class);
	public DigitalView(IModel model) {
		this.model = model;
		
		setTitle(TITEL);
		setSize(BREITE, HHE);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
		super.add(new javax.swing.JLabel(DIGITAL));
		textField = new JTextField(10);
		textField.setEditable(false);
		super.add(textField);

		for (final Ereignis ereignis : Ereignis.values())
			if (ereignis.ordinal() >= Ereignis.EIN.ordinal() && ereignis.ordinal() <= Ereignis.UM.ordinal()) {
				// nur fr die Knopf-Steuerung-Ereignisse
				JButton knopf = new JButton(ereignis.name());
				knpfe.put(ereignis, knopf);
				super.add(knopf);
			}
		knpfe.get(Ereignis.EIN).setEnabled(false);
		pack();		
	}
	/**
	 * Reaktion auf die Ereignisse von Control
	 * @param o Quelle des Ereignisses
	 * @param signal EIN, AUS, TICK oder INIT: keine Reaktion auf UM
	 */
	@Override // Observer
	public void update(final Observable<Ereignis> o, final Ereignis e) {
		switch (e) {
			case EIN:
				knpfe.get(Ereignis.EIN).setEnabled(false);		
				knpfe.get(Ereignis.AUS).setEnabled(true);
				break;
			case AUS: 
				knpfe.get(Ereignis.EIN).setEnabled(true);	
				knpfe.get(Ereignis.AUS).setEnabled(false);	
				break;
			case TICK: 
				Uhrzeit uhrzeit = model.getUhrzeit();
		        textField.setText(String.format("%02d:%02d:%02d", uhrzeit.getStunde(), uhrzeit.getMinute(), uhrzeit.getSekunde()));
				break;
			case INIT:
				for (final Ereignis ereignis : Ereignis.values()) {
					JButton knopf = knpfe.get(ereignis);
					if (knopf != null)			
						knopf.addActionListener(new ActionListener() {
							@Override
							public void actionPerformed(ActionEvent e) {
								o.notifyObservers(ereignis);
							}
						});
				}
		}
	}
}
