package genObserver;

import java.util.Calendar;

/**
 * Uhr mit MVC Oberserver
 * Model = Observer, lauscht auf die Eregnisse vom Control
 * @author Prof. Solymosi, Beuth-Hochschule fr Technik Berlin
 * @version 19.4.2012
 */
public class Model implements IModel { // Observer
	private static final long FREQUENZ = 1000, // 1 sec
		FREQUENZ_UM = 3000; // 3 sec	

		private boolean uhrAn;
		private Uhrzeit uhrzeit;
		private long frequenz;

	/**
	 * Model ist autonom (ohne Parameter)
	 */
	public Model() {
		uhrAn = true;
		frequenz = FREQUENZ;
	}
	private void tick() {
		if (uhrAn) {
			Calendar kalender = Calendar.getInstance(); 
			uhrzeit = new Uhrzeit(kalender.get(Calendar.HOUR), kalender.get(Calendar.MINUTE), kalender.get(Calendar.SECOND));
		}
	}
	@Override // IModel
	public Uhrzeit getUhrzeit() {
		return uhrzeit;
	}
	@Override // IModel
	public long getFrequenz() {
		return frequenz;
	}
	/**
	 * Reaktion auf die Ereignisse von Control
	 * @param o wird nicht benutzt
	 * @param signal EIN, AUS, TICK oder UM: keine Reaktion auf INIT
	 */
	@Override // Observer
	public void update(Observable<Ereignis> o, Ereignis signal) {
		switch (signal) {
			case EIN: uhrAn = true; break; 
			case AUS: uhrAn = false; break;
			case UM: frequenz = frequenz == FREQUENZ ? FREQUENZ_UM : FREQUENZ; break; 
			case INIT: case TICK: tick(); // uhrzeit wird aktualisiert
		}
	}
}
