package mvcObserver;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JTextField;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import mvcObserver.IModel.Uhrzeit;

/**
 * Uhr mit MVC Observer - View fr digitale Anzeige
 * Ist Observer, lauscht auf die Ereignisse vom Control
 * @author Prof. Solymosi, Beuth-Hochschule fr Technik Berlin
 * @version 16.4.2012
 */

@SuppressWarnings("serial")
public class DigitalView extends JFrame implements Observer {
	final static String DIGITAL = "Digitaluhr: "; // Beschriftung		
	final static String TITEL = "Uhr", KNOPF_EIN = "Ein", KNOPF_AUS = "Aus", KNOPF_UM = "Um";
	private static final int BREITE = 400, HHE = 300;	// Gre des Frames

	private JTextField textField;
	private JButton[] knpfe; // knopfEin, knopfAus, knopfUm;

	private IModel model;	
	public DigitalView(IModel model) {
		this.model = model;
		
		setTitle(TITEL);
		setSize(BREITE, HHE);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
		super.add(new javax.swing.JLabel(DIGITAL));
		textField = new JTextField(10);
		textField.setEditable(false);
		super.add(textField);

		knpfe = new JButton[3];
		knpfe[0] = new JButton(KNOPF_EIN);
		knpfe[1] = new JButton(KNOPF_AUS);
		knpfe[2] = new JButton(KNOPF_UM);
		super.add(knpfe[0]);
		super.add(knpfe[1]);
		super.add(knpfe[2]);		

		knpfe[0].setEnabled(false);
		pack();		
	}
	/**
	 * Reaktion auf die Ereignisse von Control
	 */	
	@Override
	public void update(Observable subjekt, Object arg) {
		if (arg.getClass().equals(String.class)) { // Oberflchenereignis
			String signal = (String)arg; // hsslich
			switch (signal) {
				case "Ein":
					knpfe[0].setEnabled(false);
					knpfe[1].setEnabled(true);
					break;
				case "Aus":
					knpfe[0].setEnabled(true);
					knpfe[1].setEnabled(false);
					break;
				case "Tick":
					Uhrzeit uhrzeit = model.getUhrzeit(); // getData im Observer-Muster
			        textField.setText(String.format("%02d:%02d:%02d", uhrzeit.getStunde(), uhrzeit.getMinute(), uhrzeit.getSekunde()));
				// keine Oberflchenreaktion auf "Um" 
			}
		} else { // Initialisierung
			ActionListener lauscher = (ActionListener)arg; // gefhrlich  
			knpfe[0].addActionListener(lauscher);
			knpfe[1].addActionListener(lauscher);
			knpfe[2].addActionListener(lauscher);
		}		
	}
}
