package mvcObserver;

/**
 * Uhr mit MVC Oberserver - View fr kreisartige Anzeige
 * Ist Observer, lauscht auf die Ereignisse vom Control
 * ohne Tastatursteuerung
 * @author Prof. Solymosi, Beuth-Hochschule fr Technik Berlin
 * @version 16.4.2012
 */

import javax.swing.JFrame;
import java.awt.Color;
import java.awt.Graphics;

@SuppressWarnings("serial")
public class KreisView extends JFrame implements java.util.Observer {
	private static final String TITEL = "KreisUhr",
			INFO = "Tasten: A(usschalten), E(inschalten), U(mschalten); Takt = ";
		private static final int BREITE = 400, HHE = 300, // Gre des Frames	
			ZENTRUM_X = BREITE / 2, ZENTRUM_Y = HHE / 2, // des Kreises
			RADIUS = 100, DURCHMESSER = 2 * RADIUS, // des Kreises
			POS_INFO_X = 20, POS_INFO_Y = 40, POS_UHRZEIT = 60; // Position der Texte
		private static final int[] ZEIGERLNGE = new int[] { 5*RADIUS/10, 6*RADIUS/10, 7*RADIUS/10 }; // Lngen der 3 Zeiger
		private static final Color[] ZEIGER_FARBE = new Color[] { Color.red, Color.green, Color.blue }; // Farben der 3 Zeiger
		private static final Color HINTERGRUND_FARBE = Color.white, KREIS_FARBE = Color.black; 	

	private IModel model;	
	public KreisView(IModel model) {
		this.model = model;
		
		setTitle(TITEL);
		setSize(BREITE, HHE);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
	// Anfangspunkt fr jeden Zeiger ist Mittelpunkt des Kreises (ZENTRUM_X, ZENTRUM_Y)
	private static final int[][] endX = new int[3][60], endY = new int[3][60]; // Endpunkte der 3 Zeiger pro Minute
	// Jeder Endpunkt wird nur einmal berechnet
	private static final double zweiPi = 2 * Math.PI;
	private static final double[] KONST = new double[] { zweiPi/12, zweiPi/60, zweiPi/60 }; // fr die Berechnung der Endpunkte der 3 Zeiger

	@Override // Frame
	public void paint(Graphics g) {
		g.clearRect(0, 0, BREITE, HHE);
		g.setColor(HINTERGRUND_FARBE);
		g.fillRect(0, 0, BREITE, HHE);
		g.setColor(KREIS_FARBE);
		g.drawOval((BREITE - DURCHMESSER) / 2, (HHE - DURCHMESSER) / 2, DURCHMESSER, DURCHMESSER); // Kreis
		
		final int[] zeit = new int[] { model.getUhrzeit().getStunde(), model.getUhrzeit().getMinute(), model.getUhrzeit().getSekunde() } ;
		g.drawString(INFO + model.getFrequenz()/1000, POS_INFO_X, POS_INFO_Y);
		g.drawString(String.format("%02d:%02d:%02d", zeit[0], zeit[1], zeit[2]), POS_INFO_X, POS_UHRZEIT); // Uhrzeit digital
		for (int i = 0; i < endX.length; i++) { // fr jeden Zeiger
			final int z = zeit[i]; // Stunde, Minute oder Sekunde
			if (endX[i][z] == 0) { // wurde noch nicht berechnet
				final double grad = z * KONST[i];					
				endX[i][z] = (int)(ZENTRUM_X + ZEIGERLNGE[i] * Math.sin(grad)); 
				endY[i][z] = (int)(ZENTRUM_Y - ZEIGERLNGE[i] * Math.cos(grad));
			}
			g.setColor(ZEIGER_FARBE[i]);
			g.drawLine(ZENTRUM_X, ZENTRUM_Y, endX[i][zeit[i]], endY[i][zeit[i]]); // Zeiger			
		}
	}
	@Override // Observer
	public void update(java.util.Observable o, Object arg) {
		repaint();
	}
}
