package uhrMonolith;

/**
 * Uhr mit Anzeiger Kreise
 * @author Prof. Clecon
 * @version 1.0, 08/2011
 * @author Prof. Solymosi, Beuth-Hochschule fr Technik Berlin
 * @version 16.4.2012
 */

import javax.swing.JFrame;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Calendar;

@SuppressWarnings("serial")
public class KreisUhr extends JFrame {
	private static final String TITEL = "KreisUhr",
		INFO = "Tasten: A(usschalten), E(inschalten), U(mschalten); Takt = ";
	private static final int FREQUENZ = 1000, FREQUENZ_UM = 3000, // 1 bzw. 3 sec 
		BREITE = 400, HHE = 300, // Gre des Frames	
		ZENTRUM_X = BREITE / 2, ZENTRUM_Y = HHE / 2, // des Kreises
		RADIUS = 100, DURCHMESSER = 2 * RADIUS, // des Kreises
		POS_INFO_X = 20, POS_INFO_Y = 40, POS_UHRZEIT = 60; // Position der Texte
	private static final int[] ZEIGERLNGE = new int[] { 5*RADIUS/10, 6*RADIUS/10, 7*RADIUS/10 }; // Lngen der 3 Zeiger
	private static final Color[] ZEIGER_FARBE = new Color[] { Color.red, Color.green, Color.blue }; // Farben der 3 Zeiger
	private static final Color HINTERGRUND_FARBE = Color.white, KREIS_FARBE = Color.black; 	
	private static class Uhrzeit {
		private int stunde, minute, sekunde;	
		public Uhrzeit(int stunde, int minute, int sekunde) {
			this.stunde = stunde;
			this.minute = minute;
			this.sekunde = sekunde;
		}	
		public int getStunde() {
			return stunde;
		}	
		public int getMinute() {
			return minute;
		}	
		public int getSekunde() {
			return sekunde;
		}
	}	
	private Uhrzeit uhrzeit;
	private boolean uhrAn;
	private int frequenz;

	public KreisUhr() {
		uhrAn = true;
		frequenz = FREQUENZ;
		tick(); // uhrzeit wird initialisiert
		
		setTitle(TITEL);
		setSize(BREITE, HHE);
		setVisible(true);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		this.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent e) {
				switch (Character.toUpperCase(e.getKeyChar())) {
				case 'E': // "Ein"
					uhrAn = true;
					break;
				case 'A': // "Aus"
					uhrAn = false;
					break;
				case 'U': // "Um"
					frequenz = frequenz == FREQUENZ ? FREQUENZ_UM : FREQUENZ;
				}				
				repaint();
			}			
		});

		new Thread() {
			@Override
			public void run() {
				try {
					while (true) {
						Thread.sleep(frequenz);
						tick();
					}
				} catch (InterruptedException e) {
				}
			}
		}.start();
	}
	private void tick() {
		if (!uhrAn)
			return;
		Calendar kalender = Calendar.getInstance(); 
		uhrzeit = new Uhrzeit(kalender.get(Calendar.HOUR), kalender.get(Calendar.MINUTE), kalender.get(Calendar.SECOND));
		repaint();
	}
	// Anfangspunkt fr jeden Zeiger ist Mittelpunkt des Kreises (ZENTRUM_X, ZENTRUM_Y)
	private static final int[][] endX = new int[3][60], endY = new int[3][60]; // Endpunkte der 3 Zeiger pro Minute
	// Jeder Endpunkt wird nur einmal berechnet
	private static final double zweiPi = 2 * Math.PI;
	private static final double[] KONST = new double[] { zweiPi/12, zweiPi/60, zweiPi/60 }; // fr die Berechnung der Endpunkte der 3 Zeiger

	@Override // Frame
	public void paint(Graphics g) {
		g.clearRect(0, 0, BREITE, HHE);
		g.setColor(HINTERGRUND_FARBE);
		g.fillRect(0, 0, BREITE, HHE);
		g.setColor(KREIS_FARBE);
		g.drawOval((BREITE - DURCHMESSER) / 2, (HHE - DURCHMESSER) / 2, DURCHMESSER, DURCHMESSER); // Kreis
		
		final int[] zeit = new int[] { uhrzeit.getStunde(), uhrzeit.getMinute(), uhrzeit.getSekunde() } ;
		g.drawString(INFO + (uhrAn ? frequenz/1000 : 0), POS_INFO_X, POS_INFO_Y);
		g.drawString(String.format("%02d:%02d:%02d", zeit[0], zeit[1], zeit[2]), POS_INFO_X, POS_UHRZEIT); // Uhrzeit digital
		for (int i = 0; i < endX.length; i++) { // fr jeden Zeiger
			final int z = zeit[i]; // Stunde, Minute oder Sekunde
			if (endX[i][z] == 0) { // wurde noch nicht berechnet
				final double grad = z * KONST[i];					
				endX[i][z] = (int)(ZENTRUM_X + ZEIGERLNGE[i] * Math.sin(grad)); 
				endY[i][z] = (int)(ZENTRUM_Y - ZEIGERLNGE[i] * Math.cos(grad));
			}
			g.setColor(ZEIGER_FARBE[i]);
			g.drawLine(ZENTRUM_X, ZENTRUM_Y, endX[i][zeit[i]], endY[i][zeit[i]]); // Zeiger			
		}
	}
	/**
	 * Test
	 * @param args irrelevant
	 */
	public static void main(String[] args) {
		new KreisUhr();
	}
}
